// This file handles mapping raw template content to module IDs
// It's used to convert templates that use raw content to use module references instead

// Helper function to find the best matching module for a given content
function findBestMatchingModule(content, componentId, fieldId, modules) {
  // If content is empty, return null
  if (!content || !content.trim()) {
    return null;
  }

  // Check if there are modules for this component and field
  let matchingModules = [];
  
  for (const category in modules) {
    const categoryModules = modules[category].filter(m => 
      m.componentId === componentId && m.fieldId === fieldId
    );
    
    if (categoryModules.length > 0) {
      matchingModules = matchingModules.concat(categoryModules);
    }
  }
  
  if (matchingModules.length === 0) {
    return null;
  }
  
  // Find the most similar module based on content
  let bestMatch = null;
  let highestSimilarity = 0;
  
  matchingModules.forEach(module => {
    // Simple similarity metric: count of matching words
    const contentWords = content.toLowerCase().split(/\s+/);
    const moduleWords = module.content.toLowerCase().split(/\s+/);
    
    // Count matching words
    let matchCount = 0;
    contentWords.forEach(word => {
      if (moduleWords.includes(word)) {
        matchCount++;
      }
    });
    
    // Calculate similarity as percentage of matching words
    const similarity = matchCount / Math.max(contentWords.length, moduleWords.length);
    
    if (similarity > highestSimilarity) {
      highestSimilarity = similarity;
      bestMatch = module;
    }
  });
  
  // Only return a match if similarity is above threshold
  return highestSimilarity > 0.5 ? bestMatch : null;
}

// Convert a raw template to use module references
function convertTemplateToModules(template, modules) {
  const result = {
    id: template.id,
    title: template.title,
    description: template.description,
    moduleRefs: {}
  };
  
  // Go through each stage, component, and field
  for (const stageId in template.data.stages) {
    if (!result.moduleRefs[stageId]) {
      result.moduleRefs[stageId] = {};
    }
    
    for (const componentId in template.data.stages[stageId]) {
      if (!result.moduleRefs[stageId][componentId]) {
        result.moduleRefs[stageId][componentId] = {};
      }
      
      for (const fieldId in template.data.stages[stageId][componentId]) {
        const content = template.data.stages[stageId][componentId][fieldId];
        
        // Find best matching module
        const matchingModule = findBestMatchingModule(content, componentId, fieldId, modules);
        
        if (matchingModule) {
          // Store module reference
          result.moduleRefs[stageId][componentId][fieldId] = matchingModule.id;
        } else {
          // If no matching module, create a custom one with this content
          const customId = `custom-${template.id}-${componentId}-${fieldId}`;
          
          // Add to appropriate module category or create a new one
          const categoryKey = stageId; // Use stage ID as category
          
          if (!modules[categoryKey]) {
            modules[categoryKey] = [];
          }
          
          // Check if custom module already exists
          const existingModule = modules[categoryKey].find(m => 
            m.id === customId && m.componentId === componentId && m.fieldId === fieldId
          );
          
          if (!existingModule) {
            // Create a concise but descriptive module title
            let moduleTitle = "";
            
            // Helper function to create more meaningful component-based titles
            function getComponentBasedTitle() {
              // Get a human-readable form of the component and field IDs
              const componentName = componentId.split('_')
                .map(word => word.charAt(0).toUpperCase() + word.slice(1))
                .join(' ');
              const fieldName = fieldId.replace(/([A-Z])/g, ' $1')
                .replace(/Text$/, '')
                .replace(/Id$/, '')
                .split(/[_\s]/)
                .map(word => word.charAt(0).toUpperCase() + word.slice(1))
                .join(' ')
                .trim();
              
              // Find core theme from content
              const lines = content.split('\n').map(l => l.trim()).filter(l => l);
              
              // Try to find a descriptive keyword from the first paragraph
              const firstPara = lines[0];
              const keywords = firstPara.match(/\b(restorative|collaborative|consensus|transformative|community|circle|mediation|third-party|written|verbal|facilitated|direct|multi-channel|needs|impact|harm|assessment|visual|phased|panel|independent|detailed|criteria|values|commitments|skills)\b/gi) || [];
              
              if (keywords.length > 0) {
                const keywordTitle = keywords[0].charAt(0).toUpperCase() + keywords[0].slice(1).toLowerCase();
                return `${keywordTitle} ${fieldName || componentName}`;
              }
              
              // Check for approach/model/process keywords
              const approachMatch = content.match(/\b(approach|model|process|method|system|framework|practice)\b/i);
              if (approachMatch) {
                // Get adjectives before the approach word
                const contentBefore = content.substring(0, approachMatch.index).trim();
                const lastWords = contentBefore.split(/\s+/).slice(-2);
                if (lastWords.length > 0 && lastWords[0].length > 3) {
                  const adjective = lastWords.join(' ');
                  return `${adjective.charAt(0).toUpperCase() + adjective.slice(1)} ${approachMatch[0]}`;
                }
              }
              
              return `${componentName} ${fieldName}`.trim();
            }
            
            // Analyze content structure
            const firstSentence = content.split(/[.!?:]\s+/)[0].trim().replace(/^[^a-zA-Z0-9]*/, '');
            const lines = content.split('\n').map(l => l.trim()).filter(l => l);
            const hasNumberedList = lines.some(line => line.match(/^[0-9]\.\s+/));
            
            // Strategy 1: Look for purpose statements (often after "Our", "The", etc.)
            const purposeMatch = content.match(/\b(Our|The|This|A)\s+([^.!?:]+)(is|uses|follows|provides|ensures|helps)\s+/i);
            if (purposeMatch) {
              const purpose = purposeMatch[2].trim();
              if (purpose.length > 5 && purpose.length < 40) {
                moduleTitle = purpose.charAt(0).toUpperCase() + purpose.slice(1);
              }
            }
            
            // Strategy 2: Check for clear format identifiers in first line
            if (!moduleTitle && lines.length > 0) {
              const firstLine = lines[0];
              if (firstLine.includes('approach') || 
                  firstLine.includes('model') || 
                  firstLine.includes('process') ||
                  firstLine.includes('system')) {
                moduleTitle = firstLine;
              }
            }
            
            // Strategy 3: Title with colon format
            if (!moduleTitle && firstSentence.includes(':')) {
              const beforeColon = firstSentence.split(':')[0].trim();
              if (beforeColon.length > 5 && beforeColon.length < 40) {
                moduleTitle = beforeColon.replace(/^(The|Our|This)\s+/i, '');
              }
            }
            
            // Strategy 4: Numbered list with key concept
            if (!moduleTitle && hasNumberedList) {
              // Get the first numbered point - often a good summary
              for (let i = 0; i < lines.length && i < 5; i++) {
                const line = lines[i];
                if (line.match(/^[0-9]\.\s+/)) {
                  let pointText = line.replace(/^[0-9]\.\s+/, '');
                  
                  // Extract core concept (first 3-5 words)
                  const words = pointText.split(/\s+/);
                  if (words.length > 2) {
                    pointText = words.slice(0, Math.min(5, words.length)).join(' ');
                  }
                  
                  if (pointText.length > 5 && pointText.length < 40) {
                    moduleTitle = pointText;
                    break;
                  }
                }
              }
            }
            
            // Strategy 5: Extract key theme based on field purpose
            if (!moduleTitle) {
              // Use component and field context to generate meaningful title
              moduleTitle = getComponentBasedTitle();
            }
            
            // Cleanup and formatting
            // Remove stopwords from beginning
            moduleTitle = moduleTitle.replace(/^(The|Our|This|An|A)\s+/i, '');
            
            // Capitalize first letter
            moduleTitle = moduleTitle.charAt(0).toUpperCase() + moduleTitle.slice(1);
            
            // Ensure reasonable length
            if (moduleTitle.length > 40) {
              const words = moduleTitle.split(/\s+/);
              moduleTitle = words.slice(0, 4).join(' ');
            }
            
            // Remove any duplicate words
            const words = moduleTitle.split(/\s+/);
            const uniqueWords = [];
            for (const word of words) {
              if (!uniqueWords.includes(word)) {
                uniqueWords.push(word);
              }
            }
            moduleTitle = uniqueWords.join(' ');
            
            // If all else fails, create a descriptive title based on component and content
            if (moduleTitle.length < 5) {
              const componentKey = componentId.split('_')[0];
              const componentName = componentKey.charAt(0).toUpperCase() + componentKey.slice(1);
              
              // Find a distinctive word from content
              const contentWords = content.split(/\s+/)
                .filter(word => 
                  word.length > 4 && 
                  !['process', 'option', 'through', 'their', 'these', 'those', 'about', 'which', 'would', 'could', 'should'].includes(word.toLowerCase())
                );
              
              const distinctiveWord = contentWords[0] || 'Standard';
              moduleTitle = distinctiveWord.charAt(0).toUpperCase() + distinctiveWord.slice(1) + " " + componentName;
            }
            
            // Create new custom module with concise title
            modules[categoryKey].push({
              id: customId,
              title: moduleTitle,
              componentId: componentId,
              fieldId: fieldId,
              content: content
            });
          }
          
          result.moduleRefs[stageId][componentId][fieldId] = customId;
        }
      }
    }
  }
  
  return result;
}

// Export the mapper functions
const templateMapper = {
  findBestMatchingModule,
  convertTemplateToModules
};

// Make sure it's available globally 
window.templateMapper = templateMapper;